/*
 *$Id: filter_vector.c,v 1.4 2001/11/10 21:19:44 mulix Exp $
 */
/*
 * Rule engine - implementation of a vector of filter nodes.
 */

#include "rule_engine_private.h"
#include "mem_manager_public.h"

#include "archdep.h"
/*
 * Private functions.
 */

/* free the 'vec' part of a filter node. don't free the node itself. */
void
free_vec_of_filter_node(struct tracker_filter_node* f)
{
    assert(f);
	/* nothing to do, since a var node contains only basic data types. */
    if (f->u.vec.nodes) {
	int i;
	for (i=0; i<f->u.vec.len; i++) {
	    if (f->u.vec.nodes[i])
		free_filter_node(f->u.vec.nodes[i]);
	}
	f->u.vec.len = 0;
	f->u.vec.nodes = NULL;
    }
}

/* print a vector node. */
void
print_filter_node_vec(struct tracker_filter_node* f, unsigned int indent)
{
    int i;

    assert(f);

    print_indent(indent);
    PPRINT("vector[%d]:\n", f->u.vec.len);
    print_indent(indent);
    for (i=0; i<f->u.vec.len; i++) {
	PPRINT("f%d:\n", i);
	if (f->u.vec.nodes[i]) {
	    do_print_filter_node(f->u.vec.nodes[i], indent+INDENT_LEVEL);
	}
	else {
	    print_indent(indent);
	    PPRINT("Empty\n");
	}
	print_indent(indent);
    }
}

/*
 * Implementation of public filter building functions.
 */

/* create a vector filter node, with the given size. */
/* returns NULL if there's not enough memory.        */
filter_node_t*
make_filter_vector(int len)
{
    struct tracker_filter_node* node =
		(struct tracker_filter_node*)
			mem_alloc(sizeof(struct tracker_filter_node));
    if (node) {
	node->type = NT_VECTOR;
	node->data_type = DT_MULTI;
	node->u.vec.len = len;
	node->u.vec.nodes = mem_alloc(sizeof(filter_node_t*) * len);
	if (!node->u.vec.nodes) {
	    mem_free(node);
	    node = NULL;
	}
	else {
	    int i;
	    for (i=0; i<len; i++)
		node->u.vec.nodes[i] = NULL;
	}
    }

    return node;
}

/* inserts the given filter node into the given vector node. */
/* returns '1' on success, '0' if 'vec' is not a vector,     */
/* '-1' if index is out of range.                            */
int
insert_filter_node(filter_node_t* vec, int index, filter_node_t* f)
{
    int rc = 1;  /* assume success. */

    assert(vec);
    
    if (vec->type != NT_VECTOR) {
	rc = 0;
	goto insert_filter_node_ret;
    }
    if (index < 0 || index >= vec->u.vec.len) {
	rc = -1;
	goto insert_filter_node_ret;
    }
    if (vec->u.vec.nodes[index]) {
	/* free former value in the vector's cell, if there was any. */
	free_filter_node(vec->u.vec.nodes[index]);
    }
    vec->u.vec.nodes[index] = f;

  insert_filter_node_ret:
    return rc;
}

/* return the filter node in the given index of the given vector.    */
/* returns NULL if index is out of range, or vec is not a DT_VECTOR. */
filter_node_t*
get_filter_node_at(filter_node_t* vec, int index)
{
    filter_node_t* f = NULL;    /* assume failure. */

    assert(vec);

    if (vec->type == NT_VECTOR && index >= 0 && index < vec->u.vec.len) {
	f = vec->u.vec.nodes[index];
    }

    return f;
}

