/* 
 * $Id: sct_rules_module_tests.c,v 1.10 2001/11/29 23:15:35 mulix Exp $
 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/resource.h>
#include <errno.h>


#include "tester.h"

#include "sct_ctrl_lib/sct_ctrl_lib.h"

static unsigned int rule_id = 0;
extern sct_ctrl_t ctrl;
extern char* progname;

int open_test(void)
{
    int fd;
    if ((fd = open("tmp",O_CREAT,S_IWUSR)) == -1){
        perror("open");
        logerr ("'%s': open failed\n", __FUNCTION__);
    }
    close(fd);
    return 1;
}

int unlink_test(void)
{
    if (unlink("tmp") == -1) {
	perror("unlink");
	logerr ("'%s': unlink failed\n", __FUNCTION__);
    }

    if (link(progname, "tmplink2") == -1){
        perror("link");
        logerr ("'%s': link failed\n", __FUNCTION__);
    }

    if (unlink("tmplink2") == -1) {
        perror("unlink");
        logerr ("'%s': unlink failed\n", __FUNCTION__);
    }

    return 1;
}

int link_test(void)
{
    if (link(progname, "tmplink") == -1){
        perror("link");
        logerr ("'%s': link failed\n", __FUNCTION__);
    }
    return 1;
}
    
int chdir_test(void) 
{
    static char cwd[1024];
    getcwd(cwd, sizeof(cwd));

    if (chdir("tmpdir") == -1){
        perror("chdir");
        logerr ("'%s': chdir \"tmpdir\" failed\n", __FUNCTION__);
    } 
    else {
        if (chdir("..") == -1){
            perror("chdir");
            logerr ("'%s': chdir \"..\" failed\n", __FUNCTION__);
        } 
    }

    return 1; 
}

int chmod_test(void)
{
    if (chmod("tmplink", S_IXUSR) == -1){
        perror("chmod");
        logerr ("'%s': chmod failed\n %s\n", __FUNCTION__);
    }
    return 1;
}

int access_test(void)
{
    if (access(progname, X_OK) == -1){
        perror("access");
        logerr ("'%s': access failed\n", __FUNCTION__);
    }
    return 1;
}

int mkdir_test(void)
{
    if (mkdir("tmpdir", S_IRWXU) == -1){
        perror("mkdir");
        logerr ("'%s': mkdir failed\n", __FUNCTION__);
    }

    if (rmdir("tmpdir") == -1){
        perror("rmdir");
        logerr ("'%s': rmdir failed\n", __FUNCTION__);
    }

    if (mkdir("tmpdir", S_IRWXU) == -1){
        perror("mkdir");
        logerr ("'%s': mkdir failed\n", __FUNCTION__);
    }
    return 1;
}

int rmdir_test(void)
{
    const char location[] = "tmpdir";
    if (rmdir(location) == -1){
        perror("rmdir");
        logerr ("'%s': rmdir failed location: %s\n", 
		__FUNCTION__, location);
    }
    return 1;
}

int getpid_test(void)
{
    getpid();
    return 1;
}

int chroot_test(void)
{
    if (chroot("/") == -1) {
        perror("chroot");
        logerr ("'%s': chroot failed\n", __FUNCTION__);
    }

    return 1; 
}

int rename_test(void)
{
   if (creat("tmprename", S_IRUSR | S_IWUSR) == -1)
       perror("creat");
   if (rename("tmprename","tmp") == -1) {
       perror("rename");
       logerr ("'%s': rename failed\n", __FUNCTION__);
   }
   return 1;
}

int nice_test(void)
{
    if (nice(1) == -1) {
        perror("nice");
        logerr ("'%s': nice failed\n", __FUNCTION__);
    }
    return 1;
}           

int dup_test(void)            
{
    int fd;
    if ((fd = dup(1)) == -1) {
        perror("dup");
        logerr ("'%s': dup failed\n", __FUNCTION__);
    }
    close(fd);
    return 1;
}

int dup2_test(void)
{
    int fd, fd2;
    if ((fd = dup2(1,2)) == -1) {
        perror("dup2");
        logerr ("'%s': dup2 failed\n", __FUNCTION__);
    }
    close(fd);
    close(fd2);
    return 1; 
}



int symlink_test(void)
{
    if (symlink(progname, "tmpsymlink") == -1){
        perror("link");
        logerr ("'%s': symlink failed\n", __FUNCTION__);
    }
    return 1;
}

int getpriority_test(void)
{
    int who;
    errno = 0;
    if (getpriority(1,who) == -1 && errno != 0) {
        perror("getpriority");
        logerr ("'%s': getpriority failed\n", __FUNCTION__);
    }
    return 1;
}

int setpriority_test(void)
{
    if (setpriority(1,1,1) == -1) {
        perror("setpriority");
        logerr ("'%s': setpriority failed\n", __FUNCTION__);
    }
    return 1;
}

int lseek_test(void)
{
    int fd;
    if ((fd = open("tmp", O_CREAT, S_IWUSR)) == -1){
        perror("open");
    }
    if (lseek(fd,3,SEEK_CUR) == -1) {
	perror("lseek");
	logerr ("'%s': lseek failed\n", __FUNCTION__);
    }
    close(fd);
    return 1;
}

static void
cleanup(void)
{
    if (unlink("tmplink") == -1) {
	perror("unlink");
    }
    if (unlink("tmpsymlink") == -1) {
	perror("unlink");
    }
    if (unlink("tmp") == -1) {
        perror("unlink");
    }
}    

static void
register_module_passwd_rule(unsigned int rid)
{
    filter_node_t* f0 = make_var_indexed_val(VT_PARAM, 0, DT_STRING);
    filter_node_t* f1 = make_string_val("passwd");
    filter_node_t* f2 = make_pattern_match(f0, f1);
    action_t* a1 = make_action_log(4, NULL);
    
    if (!sct_ctrl_add_rule(ctrl, rid, 10, f2, a1, 1))
	die ("failed to upload rule '%d'\n", rid);
}

int
test_simple_syscall_usage(void)
{
    unsigned int num_tests = 100;
    int rc = fork();
    if (rc == -1)
	die("'%s': fork failed\n", __FUNCTION__);

    /* both father and child execute this */ 
    while (--num_tests){ 
	int delay = 1+(int) (10.0 * rand() /(RAND_MAX+1.0));
	creat("passwd", S_IRUSR | S_IWUSR);
	usleep(delay);
	remove("passwd");
    }
    
    if (rc == 0) /* child */
	_exit(0);

    return 1;
}

int
test_settimeofday_usage(void)
{
    unsigned int num_tests = 100;
    int rc;
    int rid = ++rule_id;

    filter_node_t* timeval_match = make_bool_val(1);
    if (!sct_ctrl_add_rule(ctrl, rid, 79, timeval_match, make_action_log(4, NULL), 1))
	die("failed to upload rule '%d'\n", rid);

    rc = fork();
    if (rc == -1)
	die("'%s': fork failed\n", __FUNCTION__);

    /* both father and child execute this */ 
    while (--num_tests){ 
	struct timeval tv = {10, 1000 - num_tests};
	int delay = 1+(int) (10.0 * rand() /(RAND_MAX+1.0));
	settimeofday(&tv, NULL);
	usleep(delay);
    }
    
    if (rc == 0) /* child */
	_exit(0);

    if (!sct_ctrl_del_rule(ctrl, rid, 79))
	die("failed to delete rule '%d'\n", rid);
    
    return 1;
}


int 
test_rule_register_deregister(void)
{
    unsigned int num_tests = 100;
    int rc;

    if (!sct_ctrl_del_all_rules(ctrl))
	die("'%s': failed to delete all module rules\n");

    rc = fork();
    if (rc == -1)
	die("'%s': fork failed\n", __FUNCTION__);

    if (rc == 0){ /* child code */
	++rule_id;
    }
    
    /* both father and child execute this */ 
    while (--num_tests){ 
	int delay = 1+(int) (5.0 * rand() /(RAND_MAX+1.0));
	unsigned int rid = (rule_id += 2);
	register_module_passwd_rule(rid);
	usleep(delay);
	if (!sct_ctrl_del_rule(ctrl, rid, 10))
	    die ("'%s': failed to delete rule\n", __FUNCTION__);
	if (rc == 0) {
	    printf("."); fflush(stdout);
	}
    }
    
    if (rc == 0) /* child */
	_exit(0);

    sct_ctrl_print_all_rules(ctrl);

    return 1;    
}

static void 
all_syscalls_register_rules(void)
{
    filter_node_t* p0 = make_var_indexed_val(VT_PARAM, 0, DT_STRING);
    filter_node_t* p1 = make_var_indexed_val(VT_PARAM, 1, DT_STRING);

    filter_node_t* prog = make_string_val(progname);
    filter_node_t* progmatch0 = make_pattern_match(p0, prog);
    /* filter_node_t* progmatch1 = make_pattern_match(p1, prog); */

    filter_node_t* tmp = make_string_val("tmp");
    filter_node_t* tmpmatch0 = make_pattern_match(p0, tmp);
    filter_node_t* tmpmatch1 = make_pattern_match(p1, tmp);

    filter_node_t* test = make_string_val("tests");
    filter_node_t* testmatch0 = make_pattern_match(p0, test);

    filter_node_t* truematch = make_bool_val(1);

    filter_node_t* intmatch0 = make_var_indexed_val(VT_PARAM, 0, DT_INT);
    filter_node_t* intmatch1 = make_int_val(1);
    filter_node_t* intmatch2 = make_equal(intmatch0, intmatch1);

    filter_node_t* matchnum0 = make_var_indexed_val(VT_PARAM, 1, DT_INT);
    filter_node_t* matchnum1 = make_int_val(3);
    filter_node_t* matchnum2 = make_equal(matchnum0, matchnum1);

    /* filter_node_t* dir = make_string_val("."); */
    /* filter_node_t* dirmatch0 = make_pattern_match(p0, dir); */

    filter_node_t* timeval_match = make_bool_val(1);

    action_t* a1 = make_action_log(4, NULL);

    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 5, tmpmatch0, a1, 1))
	die ("failed to upload rule '%d'\n", rule_id);

    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 9, tmpmatch1, a1, 1))
	die ("failed to upload rule '%d'\n", rule_id);

    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 10, tmpmatch0, a1, 1))
	die("failed to upload rule '%d'\n", rule_id);

    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 12, testmatch0, a1, 1))
	die("failed to upload rule '%d'\n", rule_id);

    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 15, tmpmatch0, a1, 1))
	die("failed to upload rule '%d'\n", rule_id);
  
    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 19, matchnum2, a1, 1))
        die("failed to upload rule '%d'\n", rule_id);

    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 20, truematch, a1, 1))
        die("failed to upload rule '%d'\n", rule_id);
  
    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 33, progmatch0, a1, 1))
	die("failed to upload rule '%d'\n", rule_id);

    /* TODO: add 'kill' rule */
    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 34, intmatch2, a1, 1))
        die("failed to upload rule '%d'\n", rule_id);

    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 38, tmpmatch1, a1, 1))
        die("failed to upload rule '%d'\n", rule_id);

    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 39, tmpmatch0, a1, 1))
	die("failed to upload rule '%d'\n", rule_id);
    
    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 40, tmpmatch0, a1, 1))
	die("failed to upload rule '%d'\n", rule_id);

    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 41, intmatch2, a1, 1))
        die("failed to upload rule '%d'\n", rule_id);

    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 61, tmpmatch0, a1, 1))
        die("failed to upload rule '%d'\n", rule_id);

    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 63, intmatch2, a1, 1))
        die("failed to upload rule '%d'\n", rule_id);
 
    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 79, timeval_match, a1, 1))
	die("failed to upload rule '%d'\n", rule_id);

    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 83, tmpmatch1, a1, 1))
        die("failed to upload rule '%d'\n", rule_id);

    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 96, intmatch2, a1, 1))
        die("failed to upload rule '%d'\n", rule_id);

    if (!sct_ctrl_add_rule(ctrl, ++rule_id, 97, intmatch2, a1, 1))
        die("failed to upload rule '%d'\n", rule_id);

    sct_ctrl_print_all_rules(ctrl);
    sleep(3);
}

static void 
all_syscalls_deregister_rules(void)
{
    if (!sct_ctrl_del_all_rules(ctrl))
	die("failed to delete all module rules");
}

int
test_all_syscalls_usage(void)
{
    typedef struct syscall_struct_t {
    unsigned int syscall_num;
    int (*ptest)(void);
    } syscall_struct;

    syscall_struct syscall_test[] =
    {
        {5, open_test}, {9, link_test}, {10, unlink_test},
        {15, chmod_test}, {19, lseek_test}, {20, getpid},
	{33, access_test}, {34, nice_test},
        {38, rename_test}, {39, mkdir_test}, {12, chdir_test},
        {61, chroot_test}, {40, rmdir_test}, {41, dup_test},
        {63, dup2_test}, {83, symlink_test}, 
        {96, getpriority_test}, {97, setpriority_test}
    };

    const unsigned int num_syscalls = sizeof(syscall_test)/sizeof(syscall_struct) ;
    unsigned int times_to_run = 100;
    unsigned int i;
    int rc;

    all_syscalls_register_rules();

    for (i = 0; i < num_syscalls; ++i){
        rc = fork();
        if (rc == -1)
            die("'%s': fork failed\n", __FUNCTION__);
        if (rc == 0)
            break;
    }
    /* parent code */
    if (rc != 0){
	/* we have as many children as there are syscalls
	   a relic from the days when each child perform 
	   only one test */
        unsigned int children = num_syscalls;
        while (children--)
            (void)wait(NULL);
	
    } else { /* child code */
        
	while (--times_to_run){
            int delay = 1+(int) (10.0 * rand() /(RAND_MAX+1.0));
            usleep(delay);

            for (i = 0; i < num_syscalls; ++i) 
            {
	        rc = syscall_test[i].ptest();
            }
	    if (times_to_run % 10 == 0) {
	        printf("."); fflush(stdout);
	    }
        }	
   	fprintf(stderr, "child '%d' finished\n", i);
        _exit(0); 
    }
    
    cleanup();  
    all_syscalls_deregister_rules();

    return 1;
}

